from enum import Enum


class RelationshipType(Enum):
    IS_A = "is"
    DO = "do"
    RELATED_TO = "is related to"
    FORM_OF = "is form of"
    PART_OF = "is part of"
    HAS_A = "has"
    USED_FOR = "used for"
    CAPABLE_OF = "capable of"
    AT_LOCATION = "at location"
    CAUSES = "causes"
    HAS_SUBEVENT = "has subevent"
    HAS_FIRST_SUBEVENT = "has first subevent"
    HAS_LAST_SUBEVENT = "has last subevent"
    HAS_PREREQUISITE = "has prerequisite"
    HAS_PROPERTY = "has property"
    MOTIVATED_BY_GOAL = "motivated by goal"
    OBSTRUCTED_BY = "obstructed by"
    DESIRES = "desires"
    CREATED_BY = "created by"
    SYNONYM = "synonym"
    ANTONYM = "antonym"
    DISTINCT_FROM = "distinct from"
    DERIVED_FROM = "derived from"
    SYMBOL_OF = "symbol of"
    DEFINED_AS = "defined as"
    MANNER_OF = "manner of"
    LOCATED_NEAR = "located near"
    HAS_CONTEXT = "has context"
    SIMILAR_TO = "similar to"
    ETYMOLOGICALLY_RELATED_TO = "etymologically related to"
    ETYMOLOGICALLY_DERIVED_FROM = "etymologically derived from"
    CAUSES_DESIRE = "causes desire"
    MADE_OF = "made of"
    RECEIVES_ACTION = "receives action"
